package com.winhc.returnmoney.enums;

import com.winhc.common.enums.CodeMsg;
import com.winhc.common.exception.CommonException;
import org.apache.commons.lang3.ObjectUtils;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @description: 字典类
 * @author: yujie
 * @date 2020.8.18 9:17
 */
public class Dict {

    public enum RefreshStatusEnum {
        /**
         *  刷新状态
         */
        REFRESH_WAIT(1, "未刷新"),
        REFRESH_ING(2, "刷新中"),
        REFRESH_SUCCESS(3, "已刷新"),
        REFRESH_FAIL(0, "刷新失败");

        private final Integer code;

        private final String msg;

        RefreshStatusEnum(Integer code, String msg) {
            this.code = code;
            this.msg = msg;
        }

        public Integer getCode() {
            return this.code;
        }

        public String getMsg() {
            return this.msg;
        }
    }

    public enum DeletedEnum {
        /**
         *  是否删除
         */
        NORMAL(0, "正常"),
        DELETE(1, "删除");

        private final Integer code;

        private final String msg;

        DeletedEnum(Integer code, String msg) {
            this.code = code;
            this.msg = msg;
        }

        public Integer getCode() {
            return this.code;
        }

        public String getMsg() {
            return this.msg;
        }

    }

    public enum AccountBillStatusEnum {
        /**
         *  账款状态
         */
        PROGRESSING(1, "进行中"),
        OVERDUE(2, "已逾期"),
        SUCCESS(3, "已收回");

        private final Integer code;

        private final String msg;

        AccountBillStatusEnum(Integer code, String msg) {
            this.code = code;
            this.msg = msg;
        }

        public Integer getCode() {
            return this.code;
        }

        public String getMsg() {
            return this.msg;
        }

        public static List<Integer> getBillStatusList() {
            return Arrays.stream(AccountBillStatusEnum.values()).map(AccountBillStatusEnum::getCode).collect(Collectors.toList());
        }

    }

    public enum DiagnosisTrendEnum {
        /**
         *  评估趋势
         */
        ASCEND(1, "上升"),
        DESCENT(2, "下降"),
        KEEP(0, "保持");

        private final Integer code;

        private final String msg;

        DiagnosisTrendEnum(Integer code, String msg) {
            this.code = code;
            this.msg = msg;
        }

        public Integer getCode() {
            return this.code;
        }

        public String getMsg() {
            return this.msg;
        }

        public static Integer getDiagnosisTrend(BigDecimal preScore, BigDecimal score) {
            if (Objects.isNull(preScore)) {
                return 0;
            }
            if (score.compareTo(preScore) > 0) {
                return DiagnosisTrendEnum.ASCEND.getCode();
            }
            if (score.compareTo(preScore) < 0) {
                return DiagnosisTrendEnum.DESCENT.getCode();
            }
            return 0;
        }

    }

    public enum DiagnosisStatusEnum {
        /**
         *  评估状态：0诊断中；1诊断完成；9诊断失败
         */
        DIAGNOSIS_ING(0, "诊断中"),
        DIAGNOSIS_SUCCESS(1, "诊断完成"),
        DIAGNOSIS_FAIL(9, "诊断失败");

        private final Integer code;

        private final String msg;

        DiagnosisStatusEnum(Integer code, String msg) {
            this.code = code;
            this.msg = msg;
        }

        public Integer getCode() {
            return this.code;
        }

        public String getMsg() {
            return this.msg;
        }
    }

    public enum ChangeTypeEnum {
        /**
         *  变更类型，增加，减少
         */
        INCREASE(1, "增加"),
        DECREASE(2, "减少");

        private final Integer code;

        private final String msg;

        ChangeTypeEnum(Integer code, String msg) {
            this.code = code;
            this.msg = msg;
        }

        public Integer getCode() {
            return this.code;
        }

        public String getMsg() {
            return this.msg;
        }

        public static Integer getMoneyCrease(BigDecimal preMoney, BigDecimal changeMoney) {
            if (ObjectUtils.anyNull(preMoney, changeMoney)) {
                return null;
            }
            if (preMoney.compareTo(changeMoney) < 0) {
                return ChangeTypeEnum.INCREASE.getCode();
            }
            if (preMoney.compareTo(changeMoney) > 0) {
                return ChangeTypeEnum.DECREASE.getCode();
            }
            return null;
        }
    }

    public enum ReceiveTypeEnum {
        /**
         *  收入类型，1应收金额，2已收金额
         */
        RECEIVABLE(1, "应收金额"),
        RECEIVED(2, "已收金额");

        private final Integer code;

        private final String msg;

        ReceiveTypeEnum(Integer code, String msg) {
            this.code = code;
            this.msg = msg;
        }

        public Integer getCode() {
            return this.code;
        }

        public String getMsg() {
            return this.msg;
        }
    }

    public enum AccountBillStageEnum {
        /**
         *  账款阶段，1未起诉，2已起诉，3已申请执行，4终结本次执行
         */
        NON_PROSECUTE(1, "未起诉"),
        PROSECUTE_ING(2, "已起诉"),
        APPLY_EXECUTE(3, "已申请执行"),
        FINAL_EXECUTE(4, "终结本次执行");

        private final Integer code;

        private final String msg;

        AccountBillStageEnum(Integer code, String msg) {
            this.code = code;
            this.msg = msg;
        }

        public Integer getCode() {
            return this.code;
        }

        public String getMsg() {
            return this.msg;
        }

        public static String getMsgByCode(Integer code) {
            for (AccountBillStageEnum e : AccountBillStageEnum.values()) {
                if (e.getCode().equals(code)) {
                    return e.getMsg();
                }
            }
            return null;
        }
    }


    /**
     * 报告类型
     */
    public enum ReportTypeEnum {
        /**
         * 报告类型
         */
        资信的调查报告(0), 案件进展报告(1), 债权穿透分析报告(2), 公司动态监测报告(3), 律师匹配报告(4),
        简易资信报告(5), 权益及风险分析报(6), 律师分析报告(7), 可执行诊断报告(8), 案件托管报告(9),
        财产信息分析报告(10), 履债评估报告(11);
        private Integer code;

        ReportTypeEnum(Integer code) {
            this.code = code;
        }

        public Integer getCode() {
            return code;
        }

        public static String getDescByCode(Integer code) {
            for (ReportTypeEnum e : ReportTypeEnum.values()) {
                if (e.getCode().equals(code)) {
                    return e.name();
                }
            }
            return null;
        }

    }

    /**
     * 被告类型
     */
    public enum LitigantTypeEnum {
        /**
         * 报告类型
         */
        自然人("0"),企业("1"),自然人无关联("2");
        private final String code;

        LitigantTypeEnum(String code) {
            this.code = code;
        }

        public String getCode() {
            return code;
        }
    }

    public enum rankStatusEnum {
        /**
         * 评分结果
         */
        待评分(0), 评分完成(1), 评分失败(3);
        private final Integer code;

        rankStatusEnum(Integer code) {
            this.code = code;
        }

        public Integer getCode() {
            return code;
        }
    }

    public enum ReadStatusEnum {
        /**
         * 读状态
         */
        未读(0),已读(1);
        private final Integer code;

        ReadStatusEnum(Integer code) {
            this.code = code;
        }

        public Integer getCode() {
            return code;
        }
    }

    public enum OverdueEnum{
        /**
         * 逾期枚举
         */
        即将逾期("OVERDUE_WEEK"),已逾期("OVERDUED"),已逾期三个月("OVERDUE_THREEMONTH"),
        已逾期半年("OVERDUE_HALFYEAR"),已逾期一年("OVERDUE_YEAR"),已逾期三年("OVERDUE_THREEYEAR"),其他("OTHER");
        private final String code;

        OverdueEnum(String code) {
            this.code = code;
        }

        public String getCode() {
            return code;
        }
    }


    public enum RemindEnum {
        /**
         * 智能提醒类型
         */
        逾期提醒,账款评级,财产线索,风险预警;
    }


    public enum UpdateTypeEnum {
        /**
         * 修改类型
         */
        END_DATE(1, "应收日期"),
        RECEIVABLE(2, "应收金额"),
        ALL(3, "账款信息");
        private final Integer code;

        private final String message;

        UpdateTypeEnum(Integer code, String message) {
            this.code = code;
            this.message = message;
        }

        public Integer getCode() {
            return code;
        }

        public String getMessage() {
            return message;
        }
    }


    public enum MsgNameEnum {
        /**
         * 模板名称
         */
        逾期APP提醒(""),逾期短信提醒(""),
        评级变动APP提醒(""),评级变动短信提醒(""),
        财产线索APP提醒(""),财产线索短信提醒(""),
        风险提醒APP提醒(""),风险提醒短信提醒(""),评级无变动APP提醒("");
        public String code;

        MsgNameEnum(String code) {
            this.code = code;
        }
    }

    public enum MsgKindEnum{
        /**
         * 消息类型
         */
        app, weixin, sms, mail, delay
    }

    public enum VIP_SIGN_ENUM {

        /**
         *  vip类型
         */
        VIP("0"), 非VIP("1");
        private final String code;

        VIP_SIGN_ENUM(String code) {
            this.code = code;
        }

        public String getCode() {
            return code;
        }

    }

    public enum REMIND_SETTING_ENUM {

        /**
         *  提醒设置类型
         */
        不需要提醒(0), 需要提醒(1);
        private final Integer code;

        REMIND_SETTING_ENUM(Integer code) {
            this.code = code;
        }

        public Integer getCode() {
            return code;
        }

    }

    public enum TIME_SLOT_ENUM {
        全部("ALL"),近30天("MONTH");

        private String code;

        TIME_SLOT_ENUM(String code) {
            this.code = code;
        }

        public String getCode() {
            return code;
        }
    }

    public enum OverdueDayTypeNum {
        /**
         *  逾期天数类型
         */
        三个月内(1),
        三至六个月(2),
        半年至一年(3),
        一年至三年(4),
        三年以上(5);

        private final Integer code;

        OverdueDayTypeNum(Integer code) {
            this.code = code;
        }

        public Integer getCode() {
            return code;
        }
    }

    public enum OverdueTipsEnum {
        /**
         *  1催款函,2快速咨询,3追欠款,4律师函服务
         */
        催款函(1, "催款函"),
        快速咨询(2, "去咨询"),
        追欠款(3, "立即前往"),
        律师函服务(4, "律师函服务");

        private final Integer code;

        private final String desc;

        OverdueTipsEnum(Integer code, String desc) {
            this.code = code;
            this.desc = desc;
        }

        public Integer getCode() {
            return code;
        }

        public String getDesc() {
            return desc;
        }
    }

    /**
     * 用户身份
     */
    public enum IdentityEnum {
        /**
         * 用户身份
         */
        商务人士("1"),认证律师("2"),认证法务("3"),其他法律人("4");
        private final String code;

        IdentityEnum(String code) {
            this.code = code;
        }

        public String getCode() {
            return code;
        }

        public static String getDescByCode(String code) {
            for (IdentityEnum e : IdentityEnum.values()) {
                if (e.getCode().equals(code)) {
                    return e.name();
                }
            }
            // 此处未查到，返回空串
            return "";
        }
    }
}
